@@ -49,19 +49,14 @@ class ReferencesTest(utils.RepoTestCase):
                          ['refs/heads/i18n', 'refs/heads/master'])
 
         # We add a symbolic reference
-        repo.create_reference('refs/tags/version1', 'refs/heads/master',
-                              symbolic=True)
+        repo.create_reference('refs/tags/version1', 'refs/heads/master')
         self.assertEqual(sorted(repo.listall_references()),
                          ['refs/heads/i18n', 'refs/heads/master',
                           'refs/tags/version1'])
 
-        # Now we list only the symbolic references
-        self.assertEqual(repo.listall_references(GIT_REF_SYMBOLIC),
-                         ('refs/tags/version1', ))
-
     def test_head(self):
         head = self.repo.head
-        self.assertEqual(LAST_COMMIT, self.repo[head.oid].hex)
+        self.assertEqual(LAST_COMMIT, self.repo[head.target].hex)
 
     def test_lookup_reference(self):
         repo = self.repo
@@ -76,20 +71,20 @@ class ReferencesTest(utils.RepoTestCase):
 
     def test_reference_get_sha(self):
         reference = self.repo.lookup_reference('refs/heads/master')
-        self.assertEqual(reference.hex, LAST_COMMIT)
+        self.assertEqual(reference.target.hex, LAST_COMMIT)
 
 
     def test_reference_set_sha(self):
         NEW_COMMIT = '5ebeeebb320790caf276b9fc8b24546d63316533'
         reference = self.repo.lookup_reference('refs/heads/master')
-        reference.oid = NEW_COMMIT
-        self.assertEqual(reference.hex, NEW_COMMIT)
+        reference.target = NEW_COMMIT
+        self.assertEqual(reference.target.hex, NEW_COMMIT)
 
     def test_reference_set_sha_prefix(self):
         NEW_COMMIT = '5ebeeebb320790caf276b9fc8b24546d63316533'
         reference = self.repo.lookup_reference('refs/heads/master')
-        reference.oid = NEW_COMMIT[0:6]
-        self.assertEqual(reference.hex, NEW_COMMIT)
+        reference.target = NEW_COMMIT[0:6]
+        self.assertEqual(reference.target.hex, NEW_COMMIT)
 
 
     def test_reference_get_type(self):
@@ -123,10 +118,8 @@ class ReferencesTest(utils.RepoTestCase):
         # Access the deleted reference
         self.assertRaises(GitError, getattr, reference, 'name')
         self.assertRaises(GitError, getattr, reference, 'type')
-        self.assertRaises(GitError, getattr, reference, 'oid')
-        self.assertRaises(GitError, setattr, reference, 'oid', LAST_COMMIT)
-        self.assertRaises(GitError, getattr, reference, 'hex')
         self.assertRaises(GitError, getattr, reference, 'target')
+        self.assertRaises(GitError, setattr, reference, 'target', LAST_COMMIT)
         self.assertRaises(GitError, setattr, reference, 'target', "a/b/c")
         self.assertRaises(GitError, reference.delete)
         self.assertRaises(GitError, reference.resolve)
@@ -159,7 +152,7 @@ class ReferencesTest(utils.RepoTestCase):
         self.assertEqual(reference.type, GIT_REF_SYMBOLIC)
         reference = reference.resolve()
         self.assertEqual(reference.type, GIT_REF_OID)
-        self.assertEqual(reference.hex, LAST_COMMIT)
+        self.assertEqual(reference.target.hex, LAST_COMMIT)
 
 
     def test_reference_resolve_identity(self):
@@ -175,36 +168,35 @@ class ReferencesTest(utils.RepoTestCase):
         refs = self.repo.listall_references()
         self.assertTrue('refs/tags/version1' in refs)
         reference = self.repo.lookup_reference('refs/tags/version1')
-        self.assertEqual(reference.hex, LAST_COMMIT)
-        self.assertEqual(reference.target, LAST_COMMIT)
+        self.assertEqual(reference.target.hex, LAST_COMMIT)
 
         # try to create existing reference
         self.assertRaises(ValueError, self.repo.create_reference,
                           'refs/tags/version1', LAST_COMMIT)
 
         # try to create existing reference with force
-        reference =  self.repo.create_reference('refs/tags/version1',
-                        LAST_COMMIT, force=True)
-        self.assertEqual(reference.hex, LAST_COMMIT)
+        reference = self.repo.create_reference('refs/tags/version1',
+                                               LAST_COMMIT, force=True)
+        self.assertEqual(reference.target.hex, LAST_COMMIT)
 
 
     def test_create_symbolic_reference(self):
+        repo = self.repo
         # We add a tag as a new symbolic reference that always points to
         # "refs/heads/master"
-        reference = self.repo.create_reference('refs/tags/beta',
-                        'refs/heads/master', symbolic=True)
+        reference = repo.create_reference('refs/tags/beta',
+                                          'refs/heads/master')
         self.assertEqual(reference.type, GIT_REF_SYMBOLIC)
         self.assertEqual(reference.target, 'refs/heads/master')
 
 
         # try to create existing symbolic reference
-        self.assertRaises(ValueError, self.repo.create_reference,
-                          'refs/tags/beta', 'refs/heads/master',
-                          symbolic=True)
+        self.assertRaises(ValueError, repo.create_reference,
+                          'refs/tags/beta', 'refs/heads/master')
 
         # try to create existing symbolic reference with force
-        reference =  self.repo.create_reference('refs/tags/beta',
-                        'refs/heads/master', force=True, symbolic=True)
+        reference = repo.create_reference('refs/tags/beta',
+                                          'refs/heads/master', force=True)
         self.assertEqual(reference.type, GIT_REF_SYMBOLIC)
         self.assertEqual(reference.target, 'refs/heads/master')
 
@@ -213,5 +205,11 @@ class ReferencesTest(utils.RepoTestCase):
 #       self.repo.packall_references()
 
 
+    def test_get_object(self):
+        repo = self.repo
+        ref = repo.lookup_reference('refs/heads/master')
+        self.assertEqual(repo[ref.target].oid, ref.get_object().oid)
+
+
 if __name__ == '__main__':
     unittest.main()
