@@ -69,13 +69,11 @@ class WalkerTest(utils.RepoTestCase):
 
     def test_walk(self):
         walker = self.repo.walk(log[0], GIT_SORT_TIME)
-        out = [ x.hex for x in walker ]
-        self.assertEqual(out, log)
+        self.assertEqual([x.hex for x in walker], log)
 
     def test_reverse(self):
         walker = self.repo.walk(log[0], GIT_SORT_TIME | GIT_SORT_REVERSE)
-        out = [ x.hex for x in walker ]
-        self.assertEqual(out, list(reversed(log)))
+        self.assertEqual([x.hex for x in walker], list(reversed(log)))
 
     def test_hide(self):
         walker = self.repo.walk(log[0], GIT_SORT_TIME)
@@ -90,23 +88,20 @@ class WalkerTest(utils.RepoTestCase):
     def test_reset(self):
         walker = self.repo.walk(log[0], GIT_SORT_TIME)
         walker.reset()
-        out = [ x.hex for x in walker ]
-        self.assertEqual(out, [])
+        self.assertEqual([x.hex for x in walker], [])
 
     def test_push(self):
         walker = self.repo.walk(log[-1], GIT_SORT_TIME)
-        out = [ x.hex for x in walker ]
-        self.assertEqual(out, log[-1:])
+        self.assertEqual([x.hex for x in walker], log[-1:])
         walker.reset()
         walker.push(log[0])
-        out = [ x.hex for x in walker ]
-        self.assertEqual(out, log)
+        self.assertEqual([x.hex for x in walker], log)
 
     def test_sort(self):
         walker = self.repo.walk(log[0], GIT_SORT_TIME)
         walker.sort(GIT_SORT_TIME | GIT_SORT_REVERSE)
-        out = [ x.hex for x in walker ]
-        self.assertEqual(out, list(reversed(log)))
+        self.assertEqual([x.hex for x in walker], list(reversed(log)))
+
 
 if __name__ == '__main__':
     unittest.main()
